
#' Region navn og region kode for gitt år
#'
#' Denne funksjonen tar regionkoder og regionnavn fra en regional statistikk,
#' tar inn tabell som viser historiske endringer i regional inndeling av Norge
#' og gjør om til riktige koder og navn for et gitt referanseår (ref.yr)
#' Funksjonen fungerer for fylkesnivå inkludert landet ELLER for kommunenivå.
#'
#' @param regionstat is the regional statistics, where each obs represent one yar
#' and one region (fylke eller kommune)
#' @param ref.yr is the reference year to which geographic name and code is to be used
#' @param reg_level is the region level in the regionstat ("fylke"  | kommune")
#'
#' @return tibble having the regional statistics including the regional
#' names and codes for the reference year in question
#' @export
#'
#' @examples regnavn.at.ref.yr(regionstat = t12750(), ref.yr = 2020 ) %>% glimpse()
regnavn.at.ref.yr <- function(regionstat, ref.yr = year(now()), reg_level = "fylke"){
  #    regionstat = t12750() #for testing


  # Fetch the relevant region reference table regref
  if (reg_level == "fylke") { regref <- regref_fylke
  } else {regref <- regref_kommune}

  # harmonizing
  ref.yr = as.integer(ref.yr)
print("")
print("Regref:")
glimpse(regref)
 # Fetching from regref
  regref <- as.data.frame(regref, stringsAsFactors = F)
  regref_n <- names(regref) #their column names
  regref_yr <- as.integer(stringr::str_extract(names(regref), "\\d{4}")) #year
  regref_typ <-  stringr::str_sub(regref_n, 5, 8) # type (code / name)
  print("regref_yr and regref_typ wrapped in tibble:")
  glimpse(tibble(regref_yr = regref_yr, regref_typ = regref_typ))

  print("Samples from regionstat before adding regref_ref.yr_col_code:")
glimpse(regionstat[seq.int(1, dim(regionstat)[1], length.out = 10), ])

  regionstat <-
    regionstat %>%
    dplyr::mutate(.,
                  # for each obs in regionstat: tag which column in regreft one should fetch the name and code
                  #  when to fit with the ref.yr.
                  regref_ref.yr_col_code =
                    max( which(regref_yr <= ref.yr & regref_typ == "code") ),

                  regref_ref.yr_col_name =
                    max( which(regref_yr <= ref.yr &  regref_typ == "name") ))

  print("Samples from regionstat after adding regref_ref.yr_col_code:")
  glimpse(regionstat[seq.int(1, dim(regionstat)[1], length.out = 10), ])

  regionstat <-
    regionstat %>%
    dplyr::mutate(.,
                  # For each obsrv in the regionstat:
                  #    which column in regref the "ar" belongs to;
                  #       = ar_regref_col_rcode and ar_regref_col_rname
                  yr_regref_col_rcode = purrr::pmap_int(., .f = function(ar,  ...){
                    max( which( regref_yr <= ar & regref_typ == "code") )
                  }),

                  yr_regref_col_rname = purrr::pmap_int(., .f = function(ar, ...){
                    max( which( regref_yr <= ar & regref_typ == "name") )
                  }))
  print("Samples from regionstat after adding yr_regref_col_rcode:")
  glimpse(regionstat[seq.int(1, dim(regionstat)[1], length.out = 10), ])
  regionstat <-
    regionstat %>%
    dplyr::mutate(.,

                  #     if each regionstat$region is present in the column in regref
                  #       corresponding to the year of observation in regionstat
                  #       = recode_regref_row
                  regcode_in_regrefcol = purrr::pmap_lgl(., .f = function(region_kode, yr_regref_col_rcode, ...){
                    case_when(
                      region_kode %in% regref[, yr_regref_col_rcode] ~ T,
                      TRUE ~ F)
                  }),
                  #     and which row in regref the region_code belongs to:
                  #       = recode_regref_row
                  regcode_regref_row = purrr::pmap_int(., .f = function(region_kode, yr_regref_col_rcode, ...){
                    case_when(
                      region_kode %in% regref[, yr_regref_col_rcode] ~
                                 which( regref[, yr_regref_col_rcode] == region_kode)[1] ,
                      TRUE ~ NA_integer_)
                  })
                  ) %>%
    #

    dplyr::mutate(.,
                  valid_reg =
                    dplyr::case_when(
                      regcode_in_regrefcol | region_kode == "0" ~ T,
                      TRUE ~ F))

  print("Samples from regionstat after adding valid_reg:")
  glimpse(regionstat[seq.int(1, dim(regionstat)[1], length.out = 10), ])

  regionstat <- regionstat %>%

    dplyr::filter(., valid_reg == TRUE) %>%

#   Then we have indexes needed to pick the right row and column
#      to populate both reg_k@ref.yr and reg_n@ref.yr
    dplyr::mutate(.,
      !!sym(paste0("reg_n", ref.yr)) := coalesce(regref[cbind(regcode_regref_row, regref_ref.yr_col_name)], region),
      !!sym(paste0("reg_k", ref.yr)) := coalesce(regref[cbind(regcode_regref_row, regref_ref.yr_col_code)], region_kode)
      )


  return(regionstat)

}






#' Region navn og region kode for gitt år v2
#'
#' Denne funksjonen tar regionkoder og regionnavn fra en regional statistikk,
#' tar inn tabell som viser historiske endringer i regional inndeling av Norge
#' og gjør om til riktige koder og navn for et gitt referanseår (ref.yr)
#' Funksjonen fungerer for fylkesnivå inkludert landet ELLER for kommunenivå.
#'
#' @param regionstat is the regional statistics, where each obs represent one yar
#' and one region (fylke eller kommune)
#' @param ref.yr is the reference year to which geographic name and code is to be used
#'
#' @return tibble having the regional statistics including the regional
#' names and codes for the reference year in question
#' @export
#'
#' @examples regnavn.at.ref.yr2(regionstat = t12750()) %>% glimpse()
regnavn.at.ref.yr2 <- function(regionstat){
  #    regionstat = t12750() #for testing

  # Fetch the relevant region reference table regref
  if (nchar(regionstat$region_kode[1]) == 2) {
    regref <- regref_fylke_l
    } else {regref <- regref_kommune_l }

  # harmonizing
  ref.yr <- as.integer(year(now()))

 regionstat <-
    regionstat %>%
    dplyr::mutate(.,
                  # which row in regref the region_code and observation year belongs to:
                  regrefrow = purrr::pmap_int(., .f = function(region_kode, ar, ...){
                    # 1: check if exists a part of regref where "region_code_from" == region_kode AND yfrom <= ar
                    tmprr <-
                      regref %>%
                      filter(., reg_code_from == region_kode, yfrom <= ar)

                    if (nrow(tmprr) > 0 ) {
                      rrr <-  max( which(regref$reg_code_from  == region_kode & regref$yfrom <= ar))
                    } else {
                      rrr <- NA_integer_
                    }
                    return(rrr)
                  }))  %>%
   dplyr::filter(., !is.na(regrefrow)) %>%


    #   Then we have index needed to pick the right row
    #      to populate both reg_k@ref.yr and reg_n@ref.yr
    dplyr::mutate(.,
                  !!sym(paste0("reg_n", ref.yr)) := coalesce(regref$reg_name_to[(regrefrow)], region),
                  !!sym(paste0("reg_k", ref.yr)) := coalesce(regref$reg_code_to[(regrefrow)], region_kode)
    )


  return(regionstat)

}







